"""
This module provides a way to construct the temporary database
used by ``autoimport``. 

The database is used to transfer the data between the ``autoimport``
modules: ``select``, ``order`` and ``write``.

``autoimport`` always uses a sqlite3 database as an interface but
the database can be stored in memory (fast) or on the disk 
(for huge amounts of images).

"""
import sqlite3
import tempfile
import abc

def _open_db_mem():
	return (sqlite3.connect(":memory:"), None)

def _open_db_disk():
	file = tempfile.NamedTemporaryFile()
	db = sqlite3.connect(file.name)
	return (db, file)

class AbstractTemporaryDatabase(abc.ABC):
	"""
	Abstract base class for all ``TemporaryDatabase``
	implementations. 

	**Note**: ``__init__`` must set ``self._db`` to an
		  open sqlite3 connection.
	"""
	def __init__(self):
		abc.ABC.__init__(self)
		self._db = None
	@abc.abstractmethod
	def close(self):
		pass

	def cursor(self):
		return self._db.cursor()

	def dump_db(self, file):
		for line in self._db.iterdump():
			file.write("{}\n".format(line))

	
class MemoryTemporaryDatabase(AbstractTemporaryDatabase):
	def __init__(self):
		AbstractTemporaryDatabase.__init__(self)
		self._db,_ = _open_db_mem()
	def close(self):
		self._db.close()

class DiskTemporaryDatabase(AbstractTemporaryDatabase):
	def __init__(self):
		AbstractTemporaryDatabase.__init__(self)
		db, file = _open_db_disk()
		self._db = db
		self._file = file

	def close(self):
		self._db.close()
		self._file.close()
	
		

def get_temporary_db(type_):
	"""
	Return an open ``TemporaryDatabase`` with already set up tables.

	``type_`` is either ``"mem"`` for the in-memory implementation or 
	``"disk"`` for the on-disk implementation.
	"""

	implementations = {"mem": MemoryTemporaryDatabase,
			"disk": DiskTemporaryDatabase}
	if(not type_ in implementations):
		raise ValueError("unsuppored implementation: {}".format(type_))

	impl = implementations[type_]
	instance = impl()

	cursor = instance.cursor()

	cursor.execute(
		'''CREATE TABLE FILES(
			name TEXT,
			DateTime TEXT,
			DateTimeDigitized TEXT,
			DateTimeOriginal TEXT,
			Model TEXT,
			Make TEXT,
			Software TEXT)'''
		)

	cursor.execute(
		'''CREATE TABLE DIRECTORIES(
			name TEXT)'''
		)

	cursor.execute(
		'''CREATE TABLE ASSOCIATIONS(file_id INTEGER,
			directory_id INTEGER)'''
		)

	cursor.execute(
		'''CREATE TABLE KV(key TEXT,
			value TEXT)'''
		)

	return instance
